<?php

namespace Nwb\Satellites;

use InvalidArgumentException;

class File
{
    private ?int $satelliteId;
    private string $hash;
    private int $size;
    private AccessType $accessType = AccessType::PRIVATE;

    private string $path;

    /**
     * (facultatif) Permet de conserver le nom et l'extension d'origine dans le cas où le fichier a été uploadé via un formulaire
     * auquel cas son nom ressemblera à /tmp/phpAlmjdqsf sans extension.
     * Si non renseigné, alors le nom à l'upload sera déterminé par le chemin du fichier
     */
    private ?string $fullname = null;

    public function getHash(): string
    {
        return strtoupper($this->hash);
    }

    public function getSize(): int
    {
        return $this->size;
    }

    public function getAccessType(): AccessType
    {
        return $this->accessType;
    }

    public function getPath(): string
    {
        return $this->path;
    }

    public function getFullname(): ?string
    {
        return $this->fullname;
    }

    public function setHash(string $hash): self
    {
        $this->hash = $hash;

        return $this;
    }

    public function setSize(int $size): self
    {
        $this->size = $size;

        return $this;
    }

    public function setAccessType(AccessType $accessType): self
    {
        $this->accessType = $accessType;

        return $this;
    }

    public function setPath(string $path): File
    {
        if (!file_exists($path)) {
            throw new InvalidArgumentException('File does not exist');
        }

        $this->path = $path;

        return $this;
    }

    public function setFullname(?string $fullname): self
    {
        $this->fullname = $fullname;

        return $this;
    }

    public function getSatelliteId(): ?int
    {
        return $this->satelliteId;
    }

    public function getHashSize(): string
    {
        return $this->getHash() . '_' . $this->getSize();
    }

    public function setSatelliteId(?int $satelliteId): File
    {
        $this->satelliteId = $satelliteId;

        return $this;
    }

    public function setDomain(string $domain): self
    {
        $satellite = new Satellite();
        $satellite->setDomain($domain);

        $this->setSatelliteId($satellite->getId());

        return $this;
    }

    public function getDomain(): string
    {
        $satellite = new Satellite();

        return $satellite->idToDomain($this->getSatelliteId());
    }

    public static function createFromPath(string $path): File
    {
        $file = new self();
        $file->setPath($path);

        $file->setHash(md5_file($path));
        $file->setSize(filesize($path));

        return $file;
    }
}
