<?php

namespace Nwb\Satellites;

class Satellite
{
    private const DOMAIN_LAYOUT = 'cartman{num}.st.nwb.fr';

    private string $domain;
    private bool $isOnline;
    private bool $isAcceptingFiles;

    /**
     * float est nécessaire car l'espace peut dépasser la limite d'un int.
     */
    private float $availableSpace;
    private float $availableSpaceForUpload;

    public function __construct(int $id = null)
    {
        if ($id) {
            $this->domain = $this->idToDomain($id);
        }
    }

    public function getDomain(): string
    {
        return $this->domain;
    }

    public function getIsOnline(): bool
    {
        return $this->isOnline;
    }

    public function getIsAcceptingFiles(): bool
    {
        return $this->isAcceptingFiles;
    }

    public function getAvailableSpace(): float
    {
        return $this->availableSpace;
    }

    public function getAvailableSpaceForUpload(): float
    {
        return $this->availableSpaceForUpload;
    }

    public function setDomain(string $domain): self
    {
        $this->domain = $domain;

        return $this;
    }

    public function setIsOnline(bool $isOnline): self
    {
        $this->isOnline = $isOnline;

        return $this;
    }

    public function setIsAcceptingFiles(bool $isAcceptingFiles): self
    {
        $this->isAcceptingFiles = $isAcceptingFiles;

        return $this;
    }

    public function setAvailableSpace(float $availableSpace): self
    {
        $this->availableSpace = $availableSpace;

        return $this;
    }

    public function setAvailableSpaceForUpload(float $availableSpaceForUpload): self
    {
        $this->availableSpaceForUpload = $availableSpaceForUpload;

        return $this;
    }

    public function getId(): int
    {
        preg_match_all('/\d+/', $this->domain, $matches);

        return (int) $matches[0][0];
    }

    public function idToDomain(int $id): string
    {
        return str_replace('{num}', $id, self::DOMAIN_LAYOUT);
    }

    public function getFallbackDomain(): string
    {
        return str_replace('{num}', $this->getFallbackId(), self::DOMAIN_LAYOUT);
    }

    public function getFallbackUrl(string $url): string
    {
        return str_replace($this->domain, $this->getFallbackDomain(), $url);
    }

    public function getFallbackId(): int
    {
        $id = $this->getId();

        return $id % 2 === 0 ? $id - 1 : $id + 1;
    }
}
